@@ -40,12 +40,16 @@ struct Generator::GeneratorPrivate {
     QString packageName;
     int numGenerated;
     int numGeneratedWritten;
+    QStringList instantiatedContainersNames;
+    QList<const AbstractMetaType*> instantiatedContainers;
 };
 
 Generator::Generator() : m_d(new GeneratorPrivate)
 {
     m_d->numGenerated = 0;
     m_d->numGeneratedWritten = 0;
+    m_d->instantiatedContainers = QList<const AbstractMetaType*>();
+    m_d->instantiatedContainersNames = QStringList();
 }
 
 Generator::~Generator()
@@ -68,14 +72,77 @@ bool Generator::setup(const ApiExtractor& extractor, const QMap< QString, QStrin
         if (entryFound)
             break;
     }
-
     if (entryFound)
         m_d->packageName = entryFound->name();
     else
         ReportHandler::warning("Couldn't find the package name!!");
+
+    collectInstantiatedContainers();
+
     return doSetup(args);
 }
 
+QString Generator::getSimplifiedContainerTypeName(const AbstractMetaType* type)
+{
+    if (!type->isContainer())
+        return type->cppSignature();
+    QString typeName = type->cppSignature();
+    if (type->isConstant())
+        typeName.remove(0, sizeof("const ") / sizeof(char) - 1);
+    if (type->isReference())
+        typeName.chop(1);
+    while (typeName.endsWith('*') || typeName.endsWith(' '))
+        typeName.chop(1);
+    return typeName;
+}
+
+void Generator::addInstantiatedContainers(const AbstractMetaType* type)
+{
+    if (!type)
+        return;
+    foreach (const AbstractMetaType* t, type->instantiations())
+        addInstantiatedContainers(t);
+    if (!type->typeEntry()->isContainer())
+        return;
+    QString typeName = getSimplifiedContainerTypeName(type);
+    if (!m_d->instantiatedContainersNames.contains(typeName)) {
+        m_d->instantiatedContainersNames.append(typeName);
+        m_d->instantiatedContainers.append(type);
+    }
+}
+
+void Generator::collectInstantiatedContainers(const AbstractMetaFunction* func)
+{
+    addInstantiatedContainers(func->type());
+    foreach (const AbstractMetaArgument* arg, func->arguments())
+        addInstantiatedContainers(arg->type());
+}
+
+void Generator::collectInstantiatedContainers(const AbstractMetaClass* metaClass)
+{
+    if (!metaClass->typeEntry()->generateCode())
+        return;
+    foreach (const AbstractMetaFunction* func, metaClass->functions())
+        collectInstantiatedContainers(func);
+    foreach (const AbstractMetaField* field, metaClass->fields())
+        addInstantiatedContainers(field->type());
+    foreach (AbstractMetaClass* innerClass, metaClass->innerClasses())
+        collectInstantiatedContainers(innerClass);
+}
+
+void Generator::collectInstantiatedContainers()
+{
+    foreach (const AbstractMetaFunction* func, globalFunctions())
+        collectInstantiatedContainers(func);
+    foreach (const AbstractMetaClass* metaClass, classes())
+        collectInstantiatedContainers(metaClass);
+}
+
+QList<const AbstractMetaType*> Generator::instantiatedContainers() const
+{
+    return m_d->instantiatedContainers;
+}
+
 QMap< QString, QString > Generator::options() const
 {
     return QMap<QString, QString>();
@@ -307,7 +374,7 @@ bool Generator::isObjectType(const AbstractMetaClass* metaClass)
 }
 bool Generator::isObjectType(const AbstractMetaType* metaType)
 {
-    return metaType->isObject() || metaType->isQObject();
+    return isObjectType(metaType->typeEntry());
 }
 
 bool Generator::isPointer(const AbstractMetaType* type)
@@ -317,6 +384,64 @@ bool Generator::isPointer(const AbstractMetaType* type)
             || type->isValuePointer();
 }
 
+bool Generator::isCString(const AbstractMetaType* type)
+{
+    return type->isNativePointer()
+            && type->indirections() == 1
+            && type->name() == "char";
+}
+
+bool Generator::isVoidPointer(const AbstractMetaType* type)
+{
+    return type->isNativePointer()
+            && type->indirections() == 1
+            && type->name() == "void";
+}
+
+QString Generator::getFullTypeName(const TypeEntry* type) const
+{
+    return QString("%1%2").arg(type->isCppPrimitive() ? "" : "::").arg(type->qualifiedCppName());
+}
+
+QString Generator::getFullTypeName(const AbstractMetaType* type) const
+{
+    if (isCString(type))
+        return "const char*";
+    if (isVoidPointer(type))
+        return "void*";
+    if (type->typeEntry()->isContainer())
+        return QString("::%1").arg(type->cppSignature());
+    QString typeName;
+    if (type->typeEntry()->isComplex() && type->hasInstantiations())
+        typeName = getFullTypeNameWithoutModifiers(type);
+    else
+        typeName = getFullTypeName(type->typeEntry());
+    return typeName + QString("*").repeated(type->indirections());
+}
+
+QString Generator::getFullTypeName(const AbstractMetaClass* metaClass) const
+{
+    return QString("::%1").arg(metaClass->qualifiedCppName());
+}
+
+QString Generator::getFullTypeNameWithoutModifiers(const AbstractMetaType* type) const
+{
+    if (isCString(type))
+        return "const char*";
+    if (isVoidPointer(type))
+        return "void*";
+    if (!type->hasInstantiations())
+        return getFullTypeName(type->typeEntry());
+    QString typeName = type->cppSignature();
+    if (type->isConstant())
+        typeName.remove(0, sizeof("const ") / sizeof(char) - 1);
+    if (type->isReference())
+        typeName.chop(1);
+    while (typeName.endsWith('*') || typeName.endsWith(' '))
+        typeName.chop(1);
+    return QString("::%1").arg(typeName);
+}
+
 QString Generator::minimalConstructor(const AbstractMetaType* type) const
 {
     if (!type || (type->isReference() && Generator::isObjectType(type)))
@@ -344,7 +469,12 @@ QString Generator::minimalConstructor(const AbstractMetaType* type) const
     if (type->typeEntry()->isComplex()) {
         const ComplexTypeEntry* cType = reinterpret_cast<const ComplexTypeEntry*>(type->typeEntry());
         QString ctor = cType->defaultConstructor();
-        return (ctor.isEmpty()) ? minimalConstructor(classes().findClass(cType)) : ctor;
+        if (!ctor.isEmpty())
+            return ctor;
+        ctor = minimalConstructor(classes().findClass(cType));
+        if (type->hasInstantiations())
+            ctor = ctor.replace(getFullTypeName(cType), getFullTypeNameWithoutModifiers(type));
+        return ctor;
     }
 
     return minimalConstructor(type->typeEntry());
@@ -399,9 +529,15 @@ QString Generator::minimalConstructor(const AbstractMetaClass* metaClass) const
             maxArgs = numArgs;
     }
 
+    QString qualifiedCppName = metaClass->typeEntry()->qualifiedCppName();
+    QStringList templateTypes;
+    foreach (TypeEntry* templateType, metaClass->templateArguments())
+        templateTypes << templateType->qualifiedCppName();
+    QString fixedTypeName = QString("%1<%2 >").arg(qualifiedCppName).arg(templateTypes.join(", "));
+
     // Empty constructor.
     if (maxArgs == 0)
-        return QString("::%1()").arg(metaClass->qualifiedCppName());
+        return QString("::%1()").arg(qualifiedCppName);
 
     QList<const AbstractMetaFunction*> candidates;
 
@@ -445,10 +581,8 @@ QString Generator::minimalConstructor(const AbstractMetaClass* metaClass) const
                 }
             }
 
-            if (!args.isEmpty()) {
-                return QString("::%1(%2)").arg(metaClass->qualifiedCppName())
-                                          .arg(args.join(", "));
-            }
+            if (!args.isEmpty())
+                return QString("::%1(%2)").arg(qualifiedCppName).arg(args.join(", "));
 
             candidates << ctor;
         }
@@ -472,7 +606,7 @@ QString Generator::minimalConstructor(const AbstractMetaClass* metaClass) const
             args << argValue;
         }
         if (!args.isEmpty()) {
-            return QString("::%1(%2)").arg(metaClass->qualifiedCppName())
+            return QString("::%1(%2)").arg(qualifiedCppName)
                                       .arg(args.join(", "));
         }
     }
